/*******************************************************************************
 * Copyright (C) 2018, OpenRefine contributors
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package org.openrefine.browsing.facets;

import java.util.HashMap;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonTypeInfo.Id;
import com.fasterxml.jackson.databind.DatabindContext;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.jsontype.impl.TypeIdResolverBase;
import com.fasterxml.jackson.databind.type.TypeFactory;

public class FacetConfigResolver extends TypeIdResolverBase {
	
	static final public Map<String, Class<? extends FacetConfig>> s_nameToClass =
	        new HashMap<String, Class<? extends FacetConfig>>();
    
    protected TypeFactory factory = TypeFactory.defaultInstance();
    
    public static void registerFacetConfig(String moduleName, String name, Class<? extends FacetConfig> klass) {
    	s_nameToClass.put(moduleName + "/" + name, klass);
    }

    @Override
    public Id getMechanism() {
        return Id.NAME;
    }

    @Override
    public String idFromValue(Object instance) {
        return ((FacetConfig)instance).getJsonType();
    }

    @Override
    public String idFromValueAndType(Object instance, Class<?> type) {
        return ((FacetConfig)instance).getJsonType();
    }
    
    @Override
    public JavaType typeFromId(DatabindContext context, String id) {
    	// backwards compatibility
    	if (id.indexOf('/') == -1) {
    		id = "core/"+id;
    	}
    	
    	if (s_nameToClass.containsKey(id)) {
    		return factory.constructSimpleType(s_nameToClass.get(id), new JavaType[0]);
    	} else {
    		throw new IllegalArgumentException("Unknown facet type: '"+id+"'");
    	}
    }
}
